



uabrowse -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
uacall -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
uaclient -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
uadiscover -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
uahistoryread -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
uals -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
uaread -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
uaserver -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
uasubscribe -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
uawrite -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
